(*. (c) by Richard Lang, 2003 .*)
(* theory collecting all knowledge for LinearEquations
   created by: rlang 
         date: 02.10
   changed by: rlang
   last change by: rlang
             date: 02.10.20
*)

theory LinEq imports Poly Equation begin

consts
   Solve'_lineq'_equation
             :: "[bool,real, 
		   bool list] => bool list"
               ("((Script Solve'_lineq'_equation (_ _ =))// 
                  (_))" 9)

axioms
(*-- normalize --*)
  (*WN0509 compare PolyEq.all_left "[|Not(b=!=0)|] ==> (a = b) = (a - b = 0)"*)
  all_left:          "[|Not(b=!=0)|] ==> (a=b) = (a+(-1)*b=0)"
  makex1_x:          "a^^^1  = a"  
  real_assoc_1:      "a+(b+c) = a+b+c"
  real_assoc_2:      "a*(b*c) = a*b*c"

(*-- solve --*)
  lin_isolate_add1:  "(a + b*bdv = 0) = (b*bdv = (-1)*a)"
  lin_isolate_add2:  "(a +   bdv = 0) = (  bdv = (-1)*a)"
  lin_isolate_div:   "[|Not(b=0)|] ==> (b*bdv = c) = (bdv = c / b)"

ML {*
val thy = @{theory};

val LinEq_prls = (*3.10.02:just the following order due to subterm evaluation*)
  append_rls "LinEq_prls" e_rls 
	     [Calc ("op =",eval_equal "#equal_"),
	      Calc ("Tools.matches",eval_matches ""),
	      Calc ("Tools.lhs"    ,eval_lhs ""),
	      Calc ("Tools.rhs"    ,eval_rhs ""),
	      Calc ("Poly.has'_degree'_in",eval_has_degree_in ""),
 	      Calc ("Poly.is'_polyrat'_in",eval_is_polyrat_in ""),
	      Calc ("Atools.occurs'_in",eval_occurs_in ""),    
	      Calc ("Atools.ident",eval_ident "#ident_"),
	      Thm ("not_true",num_str @{thm not_true}),
	      Thm ("not_false",num_str @{thm not_false}),
	      Thm ("and_true",num_str @{thm and_true}),
	      Thm ("and_false",num_str @{thm and_false}),
	      Thm ("or_true",num_str @{thm or_true}),
	      Thm ("or_false",num_str @{thm or_false})
              ];
(* ----- erls ----- *)
val LinEq_crls = 
   append_rls "LinEq_crls" poly_crls
   [Thm  ("real_assoc_1",num_str @{thm real_assoc_1})
    (*		
     Don't use
     Calc ("HOL.divide", eval_cancel "#divide_e"),
     Calc ("Atools.pow" ,eval_binop "#power_"),
     *)
    ];

(* ----- crls ----- *)
val LinEq_erls = 
   append_rls "LinEq_erls" Poly_erls
   [Thm  ("real_assoc_1",num_str @{thm real_assoc_1})
    (*		
     Don't use
     Calc ("HOL.divide", eval_cancel "#divide_e"),
     Calc ("Atools.pow" ,eval_binop "#power_"),
     *)
    ];

ruleset' := overwritelthy @{theory} (!ruleset',
			[("LinEq_erls",LinEq_erls)(*FIXXXME:del with rls.rls'*)
			 ]);
    
val LinPoly_simplify = prep_rls(
  Rls {id = "LinPoly_simplify", preconds = [], 
       rew_ord = ("termlessI",termlessI), 
       erls = LinEq_erls, 
       srls = Erls, 
       calc = [], 
       (*asm_thm = [],*)
       rules = [
		Thm  ("real_assoc_1",num_str @{thm real_assoc_1}),
		Calc ("op +",eval_binop "#add_"),
		Calc ("op -",eval_binop "#sub_"),
		Calc ("op *",eval_binop "#mult_"),
		(*  Dont use  
		 Calc ("HOL.divide", eval_cancel "#divide_e"),		
		 Calc ("NthRoot.sqrt",eval_sqrt "#sqrt_"),
		 *)
		Calc ("Atools.pow" ,eval_binop "#power_")
		],
       scr = EmptyScr}:rls);

ruleset' := overwritelthy @{theory} (!ruleset',
			  [("LinPoly_simplify",LinPoly_simplify)]);

(*isolate the bound variable in an linear equation; 'bdv' is a meta-constant*)
val LinEq_simplify = prep_rls(
Rls {id = "LinEq_simplify", preconds = [],
     rew_ord = ("e_rew_ord",e_rew_ord),
     erls = LinEq_erls,
     srls = Erls,
     calc = [],
     (*asm_thm = [("lin_isolate_div","")],*)
     rules = [
	      Thm("lin_isolate_add1",num_str @{thm lin_isolate_add1}), 
	      (* a+bx=0 -> bx=-a *)
	      Thm("lin_isolate_add2",num_str @{thm lin_isolate_add2}), 
	      (* a+ x=0 ->  x=-a *)
	      Thm("lin_isolate_div",num_str @{thm lin_isolate_div})    
	      (*   bx=c -> x=c/b *)  
	      ],
     scr = EmptyScr}:rls);
ruleset' := overwritelthy @{theory} (!ruleset',
			[("LinEq_simplify",LinEq_simplify)]);

(*----------------------------- problem types --------------------------------*)
(* 
show_ptyps(); 
(get_pbt ["linear","univariate","equation"]);
*)

(* ---------linear----------- *)
store_pbt
 (prep_pbt thy "pbl_equ_univ_lin" [] e_pblID
 (["linear","univariate","equation"],
  [("#Given" ,["equality e_e","solveFor v_v"]),
   ("#Where" ,["False", (*WN0509 just detected: this pbl can never be used?!?*)
               "Not( (lhs e_e) is_polyrat_in v_v)",
               "Not( (rhs e_e) is_polyrat_in v_v)",
               "((lhs e_e) has_degree_in v_v)=1",
	       "((rhs e_e) has_degree_in v_v)=1"]),
   ("#Find"  ,["solutions v_v'i'"]) 
  ],
  LinEq_prls, SOME "solve (e_e::bool, v_v)",
  [["LinEq","solve_lineq_equation"]]));

(*-------------- methods------------------------------------------------------*)
store_met
 (prep_met thy "met_eqlin" [] e_metID
 (["LinEq"],
   [],
   {rew_ord'="tless_true",rls'=Atools_erls,calc = [], srls = e_rls, prls=e_rls,
    crls=LinEq_crls, nrls=norm_Poly
    (*, asm_rls=[],asm_thm=[]*)}, "empty_script"));

(* ansprechen mit ["LinEq","solve_univar_equation"] *)
store_met
(prep_met thy "met_eq_lin" [] e_metID
 (["LinEq","solve_lineq_equation"],
   [("#Given", ["equality e_e", "solveFor v_v"]),
    ("#Where", ["Not ((lhs e_e) is_polyrat_in v_v)",
                "((lhs e_e)  has_degree_in v_v) = 1"]),
    ("#Find",  ["solutions v_v'i'"])
   ],
   {rew_ord'="termlessI", rls'=LinEq_erls, srls=e_rls, prls=LinEq_prls,
    calc=[], crls=LinEq_crls, nrls=norm_Poly},
    "Script Solve_lineq_equation (e_e::bool) (v_v::real) =                 " ^
    "(let e_e =((Try         (Rewrite      all_left           False)) @@   " ^ 
    "           (Try (Repeat (Rewrite     makex1_x            False))) @@  " ^ 
    "           (Try         (Rewrite_Set expand_binoms       False)) @@   " ^ 
    "           (Try (Repeat (Rewrite_Set_Inst [(bdv, v_v::real)]          " ^
    "                                 make_ratpoly_in    False)))    @@    " ^
    "           (Try (Repeat (Rewrite_Set LinPoly_simplify      False))))e_e;" ^
    "     e_e = ((Try (Rewrite_Set_Inst [(bdv, v_v::real)]                  " ^
    "                                     LinEq_simplify True)) @@  " ^
    "            (Repeat(Try (Rewrite_Set LinPoly_simplify     False)))) e_e " ^
    " in ((Or_to_List e_e)::bool list))"
 ));
get_met ["LinEq","solve_lineq_equation"];

*}

end

